"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getJsDocs = void 0;
const tslib_1 = require("tslib");
const typescript_1 = tslib_1.__importDefault(require("typescript"));
function getJsDocs(node) {
    const jsDocs = node.jsDoc;
    const result = [];
    if (jsDocs && jsDocs.length > 0) {
        for (const jsDoc of jsDocs) {
            if (jsDoc.tags) {
                for (const tag of jsDoc.tags) {
                    result.push(getJsDocFromTag(tag));
                }
            }
            if (typeof jsDoc.comment === 'string') {
                result.push({
                    name: '',
                    comment: jsDoc.comment,
                });
            }
            else if (jsDoc.comment) {
                for (const comment of jsDoc.comment) {
                    result.push({
                        name: '',
                        comment: comment.text,
                    });
                }
            }
        }
    }
    return result;
}
exports.getJsDocs = getJsDocs;
function getJsDocFromTag(tag) {
    let type;
    let paramName;
    let optional;
    if (tag.tagName.text === 'param' && typescript_1.default.isJSDocParameterTag(tag)) {
        type = tag.typeExpression?.type;
        paramName = tag.name.getText();
        optional = tag.isBracketed;
    }
    return {
        name: tag.tagName.text,
        type,
        paramName,
        comment: typeof tag.comment === 'string' ? tag.comment : undefined,
        optional
    };
}
