"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.templateLiteralToPattern = exports.getReferencesInType = exports.isValidReference = exports.warn = void 0;
function warn(position, stage) {
    console.warn(`Warning for ${stage}: ${position.file}:${position.line + 1}:${position.character + 1}`);
}
exports.warn = warn;
function isValidReference(declarations, referenceName) {
    for (const declaration of declarations) {
        if (declaration.kind === 'reference') {
            if (declaration.name === referenceName) {
                return true;
            }
        }
        else if (declaration.name === referenceName) {
            return true;
        }
    }
    return false;
}
exports.isValidReference = isValidReference;
function getReferencesInType(type) {
    const result = [];
    if (type.kind === 'reference') {
        result.push(type);
    }
    else if (type.kind === 'object') {
        for (const member of type.members) {
            result.push(...getReferencesInType(member.type));
        }
    }
    else if (type.kind === 'array') {
        result.push(...getReferencesInType(type.type));
    }
    return result;
}
exports.getReferencesInType = getReferencesInType;
function templateLiteralToPattern(templateLiteral) {
    let pattern = '';
    for (const t of templateLiteral) {
        if (t.kind === 'string') {
            pattern += '.*';
        }
        else if (t.kind === 'number') {
            pattern += '\\d+';
        }
        else if (t.kind === 'boolean') {
            pattern += 'true|false';
        }
        else if (t.kind === 'enum') {
            pattern += t.enums.map((e) => escapeRegex(e)).join('|');
        }
    }
    return `^${pattern}$`;
}
exports.templateLiteralToPattern = templateLiteralToPattern;
function escapeRegex(s) {
    return s.replace(/[-/\\^$*+?.()|[\]{}]/g, '\\$&');
}
